/*
 * WALTZ
 *
 * In the header file there are constants and arrays for static evaluation
 * function and Alpha-beta.
 *
 * Copyright  2012-2013 Pavel Vesely <vesely@kam.mff.cuni.cz>
 *
 * This program is distributed under the GNU General Public License.
 * See COPYING for details.
 */

#ifndef ALPHABETA_H_INCLUDED
#define ALPHABETA_H_INCLUDED

#include "ai/lib.h"
//array for hasing
#include "ai/hashedpositions.h"

// evaluation function constants
#define ZONE_OF_CONTROL_HEIGHT 1
#define THREAT_HEIGHT 1000
#define THREAT_HEIGHT_MULT 20000	// for the player on move -- he is going to win!
#define FULL_SECURE 100000
#define SECURE_TYPE 25000
#define FIELD_VALUE_MULT 2
#define ZOC_CRITICAL_SUM 8
#define STACK_OF_ALL_TYPES_BONUS 50000
#define MIN_STONES_TO_PREFER_MATERIAL 30
#define MATERIAL_VALUE_MULT 3 // 3 was better than 5
#define VALUEABLE_STACK_VALUE_DOUBLE_CAPT 50000
#define VALUEABLE_STACK_COUNT_FROM 27 // the minimal number of stacks on the board to count valueable stacks

//transposition tables
#define EXACT_VALUE 0
#define LOWER_BOUND -1
#define UPPER_BOUND 1
#define TTSIZE (1 << 19)

//history heuristics
int historyPruneMoves[BOARD_ARRAY_SIZE][BOARD_ARRAY_SIZE];	//only for first move of player
//int historyButterflyMoves[BOARD_ARRAY_SIZE][BOARD_ARRAY_SIZE];

//transposition tables
typedef struct ttEntry {
	thash hash;
	Move *bestMove1, *bestMove2;
	s32 value, valueType;	//type is EXACT_VALUE, LOWER_BOUND or UPPER_BOUND
	s32 searchDepth;
	u32 searchedNodes;
} TTEntry;
TTEntry *TranspositionTable[2 * TTSIZE];	//2* because of replacement schema Twobig

//for random selecting
typedef struct fullMovesList {
	Move *move1, *move2;
	int value;		//value that is determined by search after executing the moves
	struct fullMovesList *next;
} FullMovesList;

extern int StackHeightValue[];
extern const int StackHeightValueBeginner[];
extern const int StackByCountValue[];
extern const int CountValue[];
extern const int StackHeightNotMovable[];
extern const int PossibleMovesFromField[];
extern const int StackValueByField[][BOARD_ARRAY_SIZE];
extern const int ZOCCriticalBonus[];

// -------
// FUNCTIONS
// -------
// static evaluators
int StaticValue(void);
int CheckDoubleCaptures(int pos, int type);

// transposition table functions

TTEntry *LookupPositionInTT(void);
void AddPositionToTT(int value, int type, int searchDepth, uns searchedNodes, Move * bestMove1, Move * bestMove2);

// Alpha-beta with different enhancements
int AlphaBeta(int depth, int alpha, int beta);
int AlphaBetaPV(int depth, int alpha, int beta);
int AlphaBetaPVMO(int depth, int alpha, int beta);
int AlphaBetaMO(int depth, int alpha, int beta, Move ** m1, Move ** m2);
int AlphaBetaPVMORandom(int depth, int randomMargin);
int AlphaBetaPVMONegascout(int depth, int alpha, int beta);
int AlphaBetaPVMOHistory(int depth, int alpha, int beta);
int AlphaBetaPVMOHistoryNegascout(int depth, int alpha, int beta);
int AlphaBetaPVMOBeginner(int depth, int alpha, int beta);
int AlphaBetaPVMORandomBeginner(int depth, int randomMargin);

#endif				// ALPHABETA_H_INCLUDED
