/*
 * WALTZ
 *
 * The module init has functions for initializing arrays and variables
 * with information about the current position.
 *
 * Copyright  2012-2013 Pavel Vesely <vesely@kam.mff.cuni.cz>
 *
 * This program is distributed under the GNU General Public License.
 * See COPYING for details.
 */

#include "ai/init.h"

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <time.h>

static const int StandardBoard[] = {
	-1  ,  1  ,  1  ,  1  ,  1  , 100 , 100 , 100 , 100 ,
	-1  , -2  ,  2  ,  2  ,  2  , -1  , 100 , 100 , 100 ,
	-1  , -2  , -3  ,  3  ,  3  , -2  , -1  , 100 , 100 ,
	-1  , -2  , -3  , -1  ,  1  , -3  , -2  , -1  , 100 ,
	 1  ,  2  ,  3  ,  1  , 100 , -1  , -3  , -2  , -1  ,
	100 ,  1  ,  2  ,  3  , -1  ,  1  ,  3  ,  2  ,  1  ,
	100 , 100 ,  1  ,  2  , -3  , -3  ,  3  ,  2  ,  1  ,
	100 , 100 , 100 ,  1  , -2  , -2  , -2  ,  2  ,  1  ,
	100 , 100 , 100 , 100 , -1  , -1  , -1  , -1  ,  1
};

void CountZoneOfControl(void)
{				//could be speeded up, but it probably doesn't matter; use not often; it's slow
	DPRINT2("Counting ZOC slowly ...");
	FOR(i, 0, STONE_TYPES) {
		zoneOfControl[i] = 0;
	}
	FOR(i, 0, BOARD_ARRAY_SIZE) {
		threatenByCounts[i] = 0;
	}
	int mv = moveNumber, pl = player;
	moveNumber = 1;
	player = WHITE;
	Move *m, *tmp;
	GenerateAllMoves(&m);
	while (m != NULL) {
		if (threatenByCounts[m->to] == 0)
			zoneOfControl[board[m->to] + 3]++;
		threatenByCounts[m->to]++;

		tmp = m;
		m = m->next;
		FreeMove(tmp);
	}
	m = NULL;
	player = BLACK;
	GenerateAllMoves(&m);
	while (m != NULL) {
		if (threatenByCounts[m->to] == 0)
			zoneOfControl[board[m->to] + 3]++;
		threatenByCounts[m->to]++;
		tmp = m;
		m = m->next;
		FreeMove(tmp);
	}
	moveNumber = mv;
	player = pl;
}

void CountMaterialValue(void)
{
	DPRINT2("Counting material slowly ...");
	materialValue = 0;
	FOR(i, 0, BOARD_ARRAY_SIZE) {
		DPRINT2("%d(%d) ", board[i], stackHeights[i]);
		if (board[i] != BORDER && board[i] != EMPTY)
			materialValue += (board[i] > 0 ? 1 : -1) * StackQuality(stackHeights[i], board[i], i);
	}
}

void CountHash(void)
{
	DPRINT2("Counting hash slowly ...");
	hash = 0;
	FOR(i, 0, BOARD_ARRAY_SIZE) {
		if (board[i] != BORDER)
			hash ^= HashedPositions[i][board[i] + 3][stackHeights[i]];
	}
	if (player == BLACK)
		hash ^= 1;
}

void InitBoard(int setup)
{				//not used
	if (setup == STANDARD) {
		for (int i = 0; i < BOARD_ARRAY_SIZE; i++)
			board[i] = StandardBoard[i];
	} else if (setup == RANDOM) {
		srand(time(NULL));
		int rnd[BOARD_ARRAY_SIZE];
		for (int i = 0; i < BOARD_ARRAY_SIZE; i++) {
			rnd[i] = StandardBoard[i];
		}
		int st = TOTAL_STONES;
		for (int i = 0; i < BOARD_ARRAY_SIZE; i++) {
			if (StandardBoard[i] != BORDER) {
				int j = rand() % st, k = 0;
				while (j > -1) {
					if (rnd[k] != BORDER)
						j--;
					k++;
				}
				k--;
				st--;
				board[i] = rnd[k];
				rnd[k] = BORDER;
			} else
				board[i] = BORDER;
		}
	}
	// for every board setup, stack heights are equally 1
	for (int i = 0; i < BOARD_ARRAY_SIZE; i++) {
		if (board[i] != BORDER)
			stackHeights[i] = 1;
		else
			stackHeights[i] = 0;
	}
	for (int i = 0; i < STONE_TYPES; i++) {
		counts[i] = countsByHeight[1][i] = InitialStoneCounts[i];
		highestStack[i] = 1;
		if (i != 3) {
			for (int j = 0; j < BOARD_ARRAY_SIZE; j++) {
				if (board[j] == i) {
					highestStackPos[i] = j;
					break;
				}
			}
		}
		FOR(j,0,MAX_STACK_HEIGHT) highestStackPosCache[i][j] = -1;

	}
	player = WHITE;
	moveNumber = 1;
	turnNumber = 1;
	stoneSum = 60;
	CountZoneOfControl();
	CountMaterialValue();
	CountHash();
	value = 0;
}
