/*
 * WALTZ
 *
 * Copyright  2012-2013 Pavel Vesely <vesely@kam.mff.cuni.cz>
 *
 * In this header file there are constants for the Move Ordering and arrays
 * for possible directions that are used in the functions for generating moves.
 *
 * This program is distributed under the GNU General Public License.
 * See COPYING for details.
 */

#ifndef TZAARMOVES_H_INCLUDED
#define TZAARMOVES_H_INCLUDED

#include "ai/lib.h"

// -----------------
// CONSTANTS
// ----------------
// ordering moves
#define SORT_CAPTURE_COUNT_MULT 20
#define SORT_STACK_COUNT_MULT 3
#define SORT_HISTORY_PRUNES_MULT 20
#define SORT_STACK_BONUS 6

// for generating moves
#define DIRECTION_COUNT 6
// difference in x direction
extern const int dxs[];
// difference in y direction
extern const int dys[];


// ---------------
// FUNCTIONS -- TZAAR MOVES
// ---------------

int StackQuality(int stackHeight, int stone, int fieldTo);
void ZOCAddedStoneFrom(int field, int to);
void ZOCAddedStoneTo(int field);
void ZOCRemovingStoneFrom(int field, int to);
void ZOCRemovingStoneTo(int field);
int IsEndOfGame(void);
int IsMovePossible(Move * move);
int IsMovePossible2(Move * move, int moveNum);
void ExecuteMove(Move * move);
void RevertLastMove(void);
int FieldNameToIndex(const char *field);
const char *IndexToFieldName(int index);
void GenerateAllMoves(Move ** moves);
bool HasLegalMoves(void);
void GenerateAllMovesSorted(Move ** moves);
void GenerateAllMovesSortedMove1(Move ** moves);
void GenerateAllMovesSortedMove2(Move ** moves);
void GenerateBestMovesSorted(Move ** moves, int maxMoves);
void FreeMove(Move * move);
void FreeAllMoves(Move * move, Move * exception);
void FreeAllMovesWithoutException(Move * move);
Move *CloneMove(Move * move);

void printZOCDebug(void);
void printHighestDebug(void);

#endif				// TZAARMOVES_H_INCLUDED
